/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2009-2010 EBM WebSourcing - http://www.petalslink.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package org.ow2.petals.se.xslt;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.logging.Level;

import org.ow2.petals.component.framework.AbstractComponent;
import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.jbidescriptor.generated.Jbi;
import org.ow2.petals.component.framework.jbidescriptor.generated.Provides;
import org.ow2.petals.component.framework.su.AbstractServiceUnitManager;
import org.ow2.petals.component.framework.su.ServiceUnitDataHandler;
import org.ow2.petals.component.framework.util.StringHelper;
import org.ow2.petals.se.xslt.model.XslParameterBean;
import org.ow2.petals.se.xslt.model.XsltConfiguration;
import org.ow2.petals.se.xslt.model.XsltConfigurationHandler;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * @author Roland Naudin - EBM WebSourcing
 * @author Vincent Zurczak - EBM WebSourcing
 */
public class XsltSuManager extends AbstractServiceUnitManager {

    /**
     * The SU parameter that defines the style sheet path.
     */
    private static final String CONFIG_XSL_PATH = "stylesheet";

    /**
     * The SU parameter that defines the name of output attachments.
     */
    private static final String CONFIG_OUTPUT_ATTACHMENT_NAME = "output-attachment-name";

    /**
     * The SU parameter that defines the list of XSL parameters.
     */
    private static final String CONFIG_XSL_PARAMETERS = "xsl-parameters";

    /**
     * The SU parameter that defines a XSL parameter.
     */
    private static final String CONFIG_XSL_PARAMETER = "xsl-parameter";

    /**
     * A map associating service-unit names and XSLT configuration.
     */
    private final Map<String,XsltConfigurationHandler> suNameToXsltConfigurationHandler =
        new ConcurrentHashMap<String,XsltConfigurationHandler> ();



    /**
     * Default constructor.
     * @param component the XSLT component
     */
    public XsltSuManager( AbstractComponent component ) {
        super( component );
    }


    /*
     * (non-Javadoc)
     * @see org.ow2.petals.component.framework.su.AbstractServiceUnitManager
     * #doDeploy(java.lang.String, java.lang.String, org.ow2.petals.component.framework.jbidescriptor.generated.Jbi)
     */
    @Override
    protected void doDeploy( String serviceUnitName, String suRootPath, Jbi jbiDescriptor )
    throws PEtALSCDKException {

        // Check that there is only one Provides section in the SU
        if( jbiDescriptor.getServices().getConsumes().size() != 0 )
            throw new PEtALSCDKException( "'Consumes' sections are not supported by this component." );

        final List<Provides> providesList = jbiDescriptor.getServices().getProvides();
        if( providesList == null || providesList.size() != 1 )
            throw new PEtALSCDKException( "Only one 'Provides' section is allowed by this component." );


        // Check that the required parameters are defined
        Provides provides = providesList.get( 0 );
        final ServiceUnitDataHandler suDatahandler = getSUDataHandlerForService( provides );
        final ConfigurationExtensions extensions = suDatahandler.getConfigurationExtensions( provides );

        String xslPath = extensions.get( CONFIG_XSL_PATH );
        if( xslPath != null )
            xslPath = xslPath.trim();

        if( StringHelper.isNullOrEmpty( xslPath ))
            throw new PEtALSCDKException( "The '" + CONFIG_XSL_PATH + "' parameter is not defined or is empty." );


        // The attachment parameter is optional.
        // We simply log its absence
        String outputAttachmentName = extensions.get( CONFIG_OUTPUT_ATTACHMENT_NAME );
        if( outputAttachmentName != null )
            outputAttachmentName = outputAttachmentName.trim();

        if( StringHelper.isNullOrEmpty( outputAttachmentName )
                && this.logger != null
                && this.logger.isLoggable( Level.FINE )) {

            String msg = serviceUnitName + ": the '" + CONFIG_OUTPUT_ATTACHMENT_NAME
            + "' parameter is not defined or is empty.";
            this.logger.fine( msg );
        }


        // Get the XSL parameters
        // Get them from the JBI descriptor, the parameters being complex
        Map<String, XslParameterBean> xslParamNameToXslParamBean = new HashMap<String, XslParameterBean>();
        for (Element elt : provides.getAny()) {
            if (!elt.getNodeName().endsWith(CONFIG_XSL_PARAMETERS))
                continue;

            NodeList children = elt.getElementsByTagNameNS("*", CONFIG_XSL_PARAMETER);
            if (this.logger != null && this.logger.isLoggable(Level.FINER)) {
                String msg = serviceUnitName + ": " + children.getLength();
                String suffix;
                if (children.getLength() < 2)
                    suffix = " XSL parameter was found.";
                else
                    suffix = " XSL parameters were found.";

                this.logger.finer(msg + suffix);
            }

            for (int i = 0; i < children.getLength(); i++) {
                Element child = (Element) children.item(i);
                String name = child.getAttribute("name");
                boolean overridable = Boolean.valueOf(child.getAttribute("overridable"));

                String value = null;
                if (child.getTextContent() != null)
                    value = child.getTextContent().trim();

                if (!StringHelper.isNullOrEmpty(name) && !StringHelper.isNullOrEmpty(value)) {
                    XslParameterBean xslParamBean = new XslParameterBean(name, value, overridable);
                    xslParamNameToXslParamBean.put(name, xslParamBean);

                    if (this.logger != null && this.logger.isLoggable(Level.FINER)) {
                        String msg = serviceUnitName + ": a XSL parameter was found (" + name
                        + " = " + value + ").";
                        this.logger.finer(msg);
                    }
                }
            }
        }


        // Create a model bean to avoid the use of the extensions
        // Who knows if the CDK will not clean up the configuration later on...
        XsltConfiguration config = new XsltConfiguration(
                xslPath,
                suRootPath,
                outputAttachmentName,
                provides.getEndpointName(),
                serviceUnitName, xslParamNameToXslParamBean);


        // Create a handler and register these elements
        XsltConfigurationHandler handler = new XsltConfigurationHandler( config, this.logger );

        ((XsltSe) this.component).registerXsltConfigurationHandler( config.getEndpointName(), handler );
        this.suNameToXsltConfigurationHandler.put( serviceUnitName, handler );
    }


    /*
     * (non-Javadoc)
     * @see org.ow2.petals.component.framework.su.AbstractServiceUnitManager
     * #doStart(java.lang.String)
     */
    @Override
    protected void doStart( String serviceUnitName ) throws PEtALSCDKException {

        XsltConfigurationHandler handler = this.suNameToXsltConfigurationHandler.get( serviceUnitName );
        if( handler != null )
            handler.start( this.logger );
        else
            throw new PEtALSCDKException( "No configuration handler was found for the service-unit " + serviceUnitName + " (start)." );
    }


    /*
     * (non-Javadoc)
     * @see org.ow2.petals.component.framework.su.AbstractServiceUnitManager
     * #doStop(java.lang.String)
     */
    @Override
    protected void doStop( String serviceUnitName ) throws PEtALSCDKException {

        XsltConfigurationHandler handler = this.suNameToXsltConfigurationHandler.get( serviceUnitName );
        if( handler != null )
            handler.stop();
        else
            throw new PEtALSCDKException( "No configuration handler was found for the service-unit " + serviceUnitName + " (stop)." );
    }


    /*
     * (non-Javadoc)
     * @see org.ow2.petals.component.framework.su.AbstractServiceUnitManager
     * #doUndeploy(java.lang.String)
     */
    @Override
    protected void doUndeploy( String serviceUnitName ) throws PEtALSCDKException {
        XsltConfigurationHandler handler = this.suNameToXsltConfigurationHandler.remove( serviceUnitName );
        ((XsltSe) this.component).removeXsltConfigurationHandler( handler.getEndpointName());
    }
}
